// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO20/16AO20_Linux_2.x.x.x_GSC_DN/driver/16ao20.h $
// $Rev: 52768 $
// $Date: 2023-04-05 15:16:58 -0500 (Wed, 05 Apr 2023) $

// 16AO20: Device Driver: header file

#ifndef	__16AO20_H__
#define	__16AO20_H__

#include "gsc_common.h"
#include "gsc_pci9080.h"



// macros *********************************************************************

#define	AO20_BASE_NAME						"16ao20"

// IOCTL command codes
#define	AO20_IOCTL_REG_READ					OS_IOCTL_RW( 0, 12, gsc_reg_t)
#define	AO20_IOCTL_REG_WRITE				OS_IOCTL_W ( 1, 12, gsc_reg_t)
#define	AO20_IOCTL_REG_MOD					OS_IOCTL_W ( 2, 12, gsc_reg_t)
#define	AO20_IOCTL_QUERY					OS_IOCTL_RW( 3,  4, s32)
#define	AO20_IOCTL_INITIALIZE				OS_IOCTL   ( 4)
#define	AO20_IOCTL_AUTO_CALIBRATE			OS_IOCTL   ( 5)
#define	AO20_IOCTL_AUTO_CAL_STS				OS_IOCTL_R ( 6,  4, s32)
#define	AO20_IOCTL_TX_IO_MODE				OS_IOCTL_RW( 7,  4, s32)
#define	AO20_IOCTL_TX_IO_OVER_DATA			OS_IOCTL_RW( 8,  4, s32)
#define	AO20_IOCTL_TX_IO_OVER_FRAME			OS_IOCTL_RW( 9,  4, s32)
#define	AO20_IOCTL_TX_IO_TIMEOUT			OS_IOCTL_RW(10,  4, s32)
#define	AO20_IOCTL_BUFFER_CLEAR				OS_IOCTL   (11)
#define	AO20_IOCTL_BUFFER_MODE				OS_IOCTL_RW(12,  4, s32)
#define	AO20_IOCTL_BUFFER_OVER_DATA			OS_IOCTL_RW(13,  4, s32)
#define	AO20_IOCTL_BUFFER_OVER_FRAME		OS_IOCTL_RW(14,  4, s32)
#define	AO20_IOCTL_BUFFER_SIZE				OS_IOCTL_RW(15,  4, s32)
#define	AO20_IOCTL_BUFFER_STATUS			OS_IOCTL_R (16,  4, s32)
#define	AO20_IOCTL_BURST_ENABLE				OS_IOCTL_RW(17,  4, s32)
#define	AO20_IOCTL_BURST_READY				OS_IOCTL_R (18,  4, s32)
#define	AO20_IOCTL_BURST_TRIG_SRC			OS_IOCTL_RW(19,  4, s32)
#define	AO20_IOCTL_BURST_TRIGGER			OS_IOCTL   (20)
#define	AO20_IOCTL_CHANNEL_SEL				OS_IOCTL_RW(21,  4, s32)
#define	AO20_IOCTL_CLOCK_ENABLE				OS_IOCTL_RW(22,  4, s32)
#define	AO20_IOCTL_CLOCK_OSC_SRC			OS_IOCTL_RW(23,  4, s32)
#define	AO20_IOCTL_CLOCK_READY				OS_IOCTL_R (24,  4, s32)
#define	AO20_IOCTL_CLOCK_SRC				OS_IOCTL_RW(25,  4, s32)
#define	AO20_IOCTL_CLOCK_SW					OS_IOCTL   (26)
#define	AO20_IOCTL_DATA_FORMAT				OS_IOCTL_RW(27,  4, s32)
#define	AO20_IOCTL_GROUND_SENSE				OS_IOCTL_RW(28,  4, s32)
#define	AO20_IOCTL_IRQ_SEL					OS_IOCTL_RW(29,  4, s32)
#define	AO20_IOCTL_LOAD_READY				OS_IOCTL_R (30,  4, s32)
#define	AO20_IOCTL_LOAD_REQUEST				OS_IOCTL   (31)
#define	AO20_IOCTL_NCLK						OS_IOCTL_RW(32,  4, s32)
#define	AO20_IOCTL_NRATE					OS_IOCTL_RW(33,  4, s32)
#define	AO20_IOCTL_OUTPUT_MODE				OS_IOCTL_RW(34,  4, s32)
#define	AO20_IOCTL_SAMPLE_MODE				OS_IOCTL_RW(35,  4, s32)
#define	AO20_IOCTL_XCVR_TYPE				OS_IOCTL_RW(36,  4, s32)
#define	AO20_IOCTL_TX_IO_ABORT				OS_IOCTL_R (37,  4, s32)
#define	AO20_IOCTL_WAIT_EVENT				OS_IOCTL_RW(38, 28, gsc_wait_t)
#define	AO20_IOCTL_WAIT_CANCEL				OS_IOCTL_RW(39, 28, gsc_wait_t)
#define	AO20_IOCTL_WAIT_STATUS				OS_IOCTL_RW(40, 28, gsc_wait_t)
#define	AO20_IOCTL_EXT_CLK_IN_POL			OS_IOCTL_RW(41,  4, s32)
#define	AO20_IOCTL_EXT_TRIG_IN_POL			OS_IOCTL_RW(42,  4, s32)
#define	AO20_IOCTL_EXT_TRIG_OUT_POL			OS_IOCTL_RW(43,  4, s32)



//*****************************************************************************
// AO20_IOCTL_REG_READ
// AO20_IOCTL_REG_WRITE
// AO20_IOCTL_REG_MOD
//
#define	AO20_REG_ENCODE(s,o)				GSC_REG_ENCODE(GSC_REG_TYPE_BAR2,(s),(o))
// Parameter:	gsc_reg_t*
#define	AO20_GSC_BCR						AO20_REG_ENCODE(4,0x00)// Board Control Register
#define	AO20_GSC_CSR						AO20_REG_ENCODE(4,0x04)// Channel Selection Register
#define	AO20_GSC_SRR						AO20_REG_ENCODE(4,0x08)// Sample Rate Register
#define	AO20_GSC_BOR						AO20_REG_ENCODE(4,0x0C)// Buffer Operations Register
#define	AO20_GSC_FOR						AO20_REG_ENCODE(4,0x10)// Firmware Options Register
#define	AO20_GSC_AVR						AO20_REG_ENCODE(4,0x14)// Auto Cal Values Register
#define	AO20_GSC_ODBR						AO20_REG_ENCODE(4,0x18)// Output Data Buffer Register
#define	AO20_GSC_ACR						AO20_REG_ENCODE(4,0x1C)// Adjustable Clock Register

//*****************************************************************************
// AO20_IOCTL_QUERY
//
//	Parameter:	s32
//		Pass in a value from the list below.
//		The value returned is the answer to the query.

typedef enum
{
	AO20_QUERY_AUTO_CAL_MS,		// Max auto-cal period in ms.
	AO20_QUERY_CHANNEL_MASK,	// The channel selection mask.
	AO20_QUERY_CHANNEL_MAX,		// Maximum number of channels supported.
	AO20_QUERY_CHANNEL_QTY,		// The number of A/D channels.
	AO20_QUERY_COUNT,			// Number of query options.
	AO20_QUERY_DEVICE_TYPE,		// Value from gsc_dev_type_t
	AO20_QUERY_FIFO_SIZE,		// FIFO depth in 32-bit samples
	AO20_QUERY_FILTER,			// See filter options below.
	AO20_QUERY_FREF_DEFAULT,	// The default Fref value for the device.
	AO20_QUERY_FSAMP_MAX,		// Rate Generator maximum output rate.
	AO20_QUERY_FSAMP_MIN,		// Rate Generator minimum output rate.
	AO20_QUERY_INIT_MS,			// Max initialize period in ms.
	AO20_QUERY_NCLK_MASK,		// Mask of valid Nclk bits.
	AO20_QUERY_NCLK_MAX,		// Maximum rate divider Nclk value.
	AO20_QUERY_NCLK_MIN,		// Maximum rate divider Nclk value.
	AO20_QUERY_NRATE_MASK,		// Mask of valid Nrate bits.
	AO20_QUERY_NRATE_MAX,		// Maximum rate generator Nrate value.
	AO20_QUERY_NRATE_MIN,		// Maximum rate generator Nrate value.
	AO20_QUERY_RANGE,			// See voltage range options below.
	AO20_QUERY_EXT_CLK_IN_POL,		// BCR D18
	AO20_QUERY_EXT_TRIG_IN_POL,		// BCR D16
	AO20_QUERY_EXT_TRIG_OUT_POL,	// BCR D17
	AO20_QUERY_LAST				// Must be very last entry.

} ao20_query_t;

#define	AO20_IOCTL_QUERY_ERROR	(-1)	// A query request is unknown of invalid.

// AO20_QUERY_FILTER options.
#define	AO20_FILTER_NONE					0
#define	AO20_FILTER_10KHZ					1
#define	AO20_FILTER_100KHZ					2

// AO20_QUERY_RANGE options:
#define	AO20_RANGE_2_5						0
#define	AO20_RANGE_5						1
#define	AO20_RANGE_10						2

//*****************************************************************************
// AO20_IOCTL_INITIALIZE
//
//	Parameter:	None

//*****************************************************************************
// AO20_IOCTL_AUTO_CALIBRATE
//
//	Parameter:	None

//*****************************************************************************
// AO20_IOCTL_AUTO_CAL_STS
//
//	Parameter:	s32*
//		One of the below values is returned.
#define	AO20_AUTO_CAL_STS_ACTIVE			0
#define	AO20_AUTO_CAL_STS_FAIL				1
#define	AO20_AUTO_CAL_STS_PASS				2

//*****************************************************************************
// AO20_IOCTL_TX_IO_MODE
//
//	Parameter:	s32
//		Pass in any of the gsc_io_mode_t options, or
//		-1 to read the current setting.
#define	AO20_IO_MODE_DEFAULT				GSC_IO_MODE_PIO

//*****************************************************************************
// AO20_IOCTL_TX_IO_OVER_DATA
//
//	Parameter:	s32
//		Pass in any valid value, or
//		-1 to read the current setting.
#define	AO20_TX_IO_OVER_DATA_IGNORE			0
#define	AO20_TX_IO_OVER_DATA_CHECK			1
#define	AO20_TX_IO_OVER_DATA_DEFAULT		AO20_TX_IO_OVER_DATA_CHECK

//*****************************************************************************
// AO20_IOCTL_TX_IO_OVER_FRAME
//
//	Parameter:	s32
//		Pass in any valid value, or
//		-1 to read the current setting.
#define	AO20_TX_IO_OVER_FRAME_IGNORE		0
#define	AO20_TX_IO_OVER_FRAME_CHECK			1
#define	AO20_TX_IO_OVER_FRAME_DEFAULT		AO20_TX_IO_OVER_FRAME_CHECK

//*****************************************************************************
// AO20_IOCTL_TX_IO_TIMEOUT (in seconds)
//
//	Parameter:	s32
//		Pass in any value from the minimim to the maximim, the infinite option,
//		or -1 to read the current setting. The value -1 is returned if the
//		feature is not supported, but this should never happen.
#define	AO20_IO_TIMEOUT_DEFAULT				10
#define	AO20_IO_TIMEOUT_NO_SLEEP			0		// Do not sleep to wait for more data.
#define	AO20_IO_TIMEOUT_MIN					0
#define	AO20_IO_TIMEOUT_MAX					GSC_IO_TIMEOUT_MAX
#define	AO20_IO_TIMEOUT_INFINITE			GSC_IO_TIMEOUT_INFINITE

//*****************************************************************************
// AO20_IOCTL_BUFFER_CLEAR
//	The driver will wait the write timeout for completion.
//
//	Parameter:	None.

//*****************************************************************************
// AO20_IOCTL_BUFFER_MODE
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_BUFFER_MODE_OPEN				0
#define	AO20_BUFFER_MODE_CIRC				1

//*****************************************************************************
// AO20_IOCTL_BUFFER_OVER_DATA
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_BUFFER_OVER_DATA_CLR			0
#define	AO20_BUFFER_OVER_DATA_CHK			(-1)

//		The service returns one of the below listed values.
#define	AO20_BUFFER_OVER_DATA_NO			0
#define	AO20_BUFFER_OVER_DATA_YES			1

//*****************************************************************************
// AO20_IOCTL_BUFFER_OVER_FRAME
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_BUFFER_OVER_FRAME_CLR			0
#define	AO20_BUFFER_OVER_FRAME_CHK			(-1)

//		The service returns one of the below listed values.
#define	AO20_BUFFER_OVER_FRAME_NO			0
#define	AO20_BUFFER_OVER_FRAME_YES			1

//*****************************************************************************
// AO20_IOCTL_BUFFER_SIZE
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_BUFFER_SIZE_8					0
#define	AO20_BUFFER_SIZE_16					1
#define	AO20_BUFFER_SIZE_32					2
#define	AO20_BUFFER_SIZE_64					3
#define	AO20_BUFFER_SIZE_128				4
#define	AO20_BUFFER_SIZE_256				5
#define	AO20_BUFFER_SIZE_512				6
#define	AO20_BUFFER_SIZE_1K					7
#define	AO20_BUFFER_SIZE_2K					8
#define	AO20_BUFFER_SIZE_4K					9
#define	AO20_BUFFER_SIZE_8K					10
#define	AO20_BUFFER_SIZE_16K				11
#define	AO20_BUFFER_SIZE_32K				12
#define	AO20_BUFFER_SIZE_64K				13
#define	AO20_BUFFER_SIZE_128K				14
#define	AO20_BUFFER_SIZE_256K				15

//*****************************************************************************
// AO20_IOCTL_BUFFER_STATUS
//
//	Parameter:	s32
//		The service returns one of the below listed values.
#define	AO20_BUFFER_STATUS_EMPTY			0
#define	AO20_BUFFER_STATUS_1Q_FULL			1
#define	AO20_BUFFER_STATUS_MEDIUM			2
#define	AO20_BUFFER_STATUS_3Q_FULL			3
#define	AO20_BUFFER_STATUS_FULL				4

//*****************************************************************************
// AO20_IOCTL_BURST_ENABLE
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_BURST_ENABLE_NO				0
#define	AO20_BURST_ENABLE_YES				1

//*****************************************************************************
// AO20_IOCTL_BURST_READY
//
//	Parameter:	s32
//		The service returns one of the below listed values.
#define	AO20_BURST_READY_NO					0
#define	AO20_BURST_READY_YES				1

//*****************************************************************************
// AO20_IOCTL_BURST_TRIG_SRC
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_BURST_TRIG_SRC_EXT				0
#define	AO20_BURST_TRIG_SRC_SW				1

//*****************************************************************************
// AO20_IOCTL_BURST_TRIGGER
//	The driver will wait the write timeout for completion.
//
//	Parameter:	None.

//*****************************************************************************
// AO20_IOCTL_CHANNEL_SEL
//
//	Parameter:	s32
//		Pass in any valid bitwise combination, or
//		-1 to read the current setting.
//		The range for  6 channel devices is 0x00000 to 0x0003F.
//		The range for 12 channel devices is 0x00000 to 0x00FFF.
//		The range for 20 channel devices is 0x00000 to 0xFFFFF.

//*****************************************************************************
// AO20_IOCTL_CLOCK_ENABLE
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_CLOCK_ENABLE_NO				0
#define	AO20_CLOCK_ENABLE_YES				1

//*****************************************************************************
// AO20_IOCTL_CLOCK_OSC_SRC
//
//	Parameter:	s32
//		The service returns one of the below listed values.
#define	AO20_CLOCK_OSC_SRC_PRI				0
#define	AO20_CLOCK_OSC_SRC_ALT				1

//*****************************************************************************
// AO20_IOCTL_CLOCK_READY
//
//	Parameter:	s32
//		The service returns one of the below listed values.
#define	AO20_CLOCK_READY_NO					0
#define	AO20_CLOCK_READY_YES				1

//*****************************************************************************
// AO20_IOCTL_CLOCK_SRC
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_CLOCK_SRC_INT					0
#define	AO20_CLOCK_SRC_EXT_SW				1

//*****************************************************************************
// AO20_IOCTL_CLOCK_SW
//	The driver will wait the write timeout for completion.
//
//	Parameter:	None.

//*****************************************************************************
// AO20_IOCTL_DATA_FORMAT
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_DATA_FORMAT_2S_COMP			0	// Twos Compliment
#define	AO20_DATA_FORMAT_OFF_BIN			1	// Offset Binary

//*****************************************************************************
// AO20_IOCTL_GROUND_SENSE
//	The driver will wait the write timeout for completion.
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_GROUND_SENSE_DISABLE			0
#define	AO20_GROUND_SENSE_REMOTE			1

//*****************************************************************************
// AO20_IOCTL_IRQ_SEL
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_IRQ_INIT_DONE					0
#define	AO20_IRQ_AUTO_CAL_DONE				1
#define	AO20_IRQ_BUF_EMPTY					2
#define	AO20_IRQ_BUF_1Q_FULL				3
#define	AO20_IRQ_BUF_3Q_FULL				4
#define	AO20_IRQ_BURST_TRIG_READY			5
#define	AO20_IRQ_LOAD_READY					6
#define	AO20_IRQ_LOAD_READY_END				7

//*****************************************************************************
// AO20_IOCTL_LOAD_READY
//
//	Parameter:	s32
//		The service returns one of the below listed values.
#define	AO20_LOAD_READY_NO					0
#define	AO20_LOAD_READY_YES					1

//*****************************************************************************
// AO20_IOCTL_LOAD_REQUEST
//	The driver will wait the write timeout for completion.
//
//	Parameter:	None

//*****************************************************************************
// AO20_IOCTL_NCLK
//
//	Parameter:	s32
//		Pass in any valid value, or
//		-1 to read the current setting.
//		The valid range is 0 to 0x1FF.

//*****************************************************************************
// AO20_IOCTL_NRATE
//
//	Parameter:	s32
//		Pass in any valid value, or
//		-1 to read the current setting.
//		The valid range is 68 to 0xFFFF.

//*****************************************************************************
// AO20_IOCTL_OUTPUT_MODE
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_OUTPUT_MODE_SEQ				0
#define	AO20_OUTPUT_MODE_SIM				1

//*****************************************************************************
// AO20_IOCTL_SAMPLE_MODE
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_SAMPLE_MODE_CONT				0
#define	AO20_SAMPLE_MODE_BURST				1

//*****************************************************************************
// AO20_IOCTL_XCVR_TYPE
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO20_XCVR_TYPE_TTL					0
#define	AO20_XCVR_TYPE_LVDS					1

//*****************************************************************************
// AO20_IOCTL_TX_IO_ABORT
//
//	Parameter:	s32*
//		The returned value is one of the below options.
#define	AO20_IO_ABORT_NO					0
#define	AO20_IO_ABORT_YES					1

//*****************************************************************************
// AO20_IOCTL_WAIT_EVENT					all fields must be valid
// AO20_IOCTL_WAIT_CANCEL					fields need not be valid
// AO20_IOCTL_WAIT_STATUS					fields need not be valid
//
//	Parameter:	gsc_wait_t*
// gsc_wait_t.flags - see gsc_common.h
// gsc_wait_t.main - see gsc_common.h
// gsc_wait_t.gsc
#define	AO20_WAIT_GSC_INIT_DONE				0x0001
#define	AO20_WAIT_GSC_AUTO_CAL_DONE			0x0002
#define	AO20_WAIT_GSC_BUF_EMPTY				0x0004
#define	AO20_WAIT_GSC_BUF_1Q_FULL			0x0008
#define	AO20_WAIT_GSC_BUF_3Q_FULL			0x0010
#define	AO20_WAIT_GSC_BURST_TRIG_READY		0x0020
#define	AO20_WAIT_GSC_LOAD_READY			0x0040
#define	AO20_WAIT_GSC_LOAD_READY_END		0x0080
#define	AO20_WAIT_GSC_ALL					0x00FF
// gsc_wait_t.alt flags
#define	AO20_WAIT_ALT_ALL					0x0000
// gsc_wait_t.io
#define	AO20_WAIT_IO_TX_ABORT				0x0001
#define	AO20_WAIT_IO_TX_DONE				0x0002
#define	AO20_WAIT_IO_TX_ERROR				0x0004
#define	AO20_WAIT_IO_TX_TIMEOUT				0x0008
#define	AO20_WAIT_IO_ALL					0x000F

//*****************************************************************************
// AO20_IOCTL_EXT_CLK_IN_POL				BCR D18
// AO20_IOCTL_EXT_TRIG_IN_POL				BCR D16
// AO20_IOCTL_EXT_TRIG_OUT_POL				BCR D17
//
//	Parameter:	s32*
//		The returned value is one of the below options.
#define	AO20_EXT_POL_NORMAL					0
#define	AO20_EXT_POL_INVERT					1



#endif
