// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO20/16AO20_Linux_2.x.x.x_GSC_DN/samples/sbtest/channel_sel.c $
// $Rev: 52761 $
// $Date: 2023-04-05 15:08:50 -0500 (Wed, 05 Apr 2023) $

// 16AO20: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_CHANNEL_SEL,
			/* arg		*/	-1,
			/* reg		*/	AO20_GSC_CSR,
			/* mask		*/	-1,
			/* value	*/	-1
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_CHANNEL_SEL,
			/* arg		*/	-1,
			/* reg		*/	AO20_GSC_CSR,
			/* mask		*/	-1,
			/* value	*/	-1
		},

		{ SERVICE_END_LIST }
	};

	s32	chans;
	int	errs	= 0;
	int	i;
	u32	mask;
	u32	value;

	errs	+= ao20_query(fd, -1, 0, AO20_QUERY_CHANNEL_QTY, &chans);

	if (errs)
	{
	}
	else if ((chans != 6) && (chans != 12) && (chans != 20))
	{
		errs++;
		printf(	"FAIL <---  (channel quantity query error: %ld)\n",
				(long) chans);
	}
	else
	{
		errs			+= ao20_initialize(fd, -1, 0);
		mask			= 0xFFFFFFFF >> (32 - chans);
		list[0].mask	= mask;
		list[1].mask	= mask;

		for (i = 0; i < chans; i++)
		{
			value			= 0x1L << i;
			list[0].arg		= value;
			list[0].value	= value;
			list[1].arg		= mask ^ value;
			list[1].value	= mask ^ value;

			errs	+= service_ioctl_set_reg_list(fd, list);
			errs	+= service_ioctl_reg_get_list(fd, list);

			errs	+= service_ioctl_set_reg_list(fd, list);
			errs	+= service_ioctl_reg_get_list(fd, list);
		}

		errs	+= ao20_initialize(fd, -1, 0);
	}

	return(errs);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{					// Select DMA mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_BMDMA,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select simultaneous outputs.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_OUTPUT_MODE,
			/* arg		*/	AO20_OUTPUT_MODE_SIM,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Set Nrate high for the lowest possible Fsamp.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_NRATE,
			/* arg		*/	0xFFFF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select the 30MHz reference source.
			/* cmd		*/	SERVICE_NORMAL,
			/* arg		*/	AO20_IOCTL_CLOCK_OSC_SRC,
			/* reg		*/	AO20_CLOCK_OSC_SRC_PRI,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{					// Enable two channels.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_CHANNEL_SEL,
			/* arg		*/	0x3,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{					// Enable one channel only.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_CHANNEL_SEL,
			/* arg		*/	0x1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	clock_enable[]	=
	{
		{					// Enable the clock.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AO20_CLOCK_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	clock_disable[]	=
	{
		{					// Clear the buffer and flags.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_BUFFER_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Disable the clock.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO20_IOCTL_CLOCK_ENABLE,
			/* arg		*/	AO20_CLOCK_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	#define	SAMPLES	1602

	s32	_1_ne;
	s32	_2_e;
	s32	_3_ne;
	s32	_4_e;
	u32	buf[SAMPLES];
	int	errs	= 0;
	int	ret;
	int	sent;

	// The procedure is to select a sample rate and duration so we can look at
	// the time it takes for the buffer to empty with both 2 channels enabled
	// and only one channel enabled.

	memset(buf, 0, sizeof(buf));
	errs	+= ao20_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list_1);

	// Here, it should take about 1.75 seconds for the buffer to empty.
	errs	+= service_ioctl_set_reg_list(fd, list_2);
	errs	+= service_ioctl_set_reg_list(fd, clock_disable);
	sent	= ao20_write(fd, buf, sizeof(buf));

	if (sent != sizeof(buf))
	{
		errs++;
		printf(	"ERROR: ao20_write(): send %ld, expected %ld)\n",
				(long) sent,
				(long) sizeof(buf));
	}

	errs	+= service_ioctl_set_reg_list(fd, clock_enable);
	os_sleep_ms(1000);
	ret		= ao20_ioctl(fd, AO20_IOCTL_BUFFER_STATUS,&_1_ne);
	errs	+= ret ? 1 : 0;
	os_sleep_ms(1000);
	ret		= ao20_ioctl(fd, AO20_IOCTL_BUFFER_STATUS,&_2_e);
	errs	+= ret ? 1 : 0;
	errs	+= service_ioctl_set_reg_list(fd, clock_disable);

	// Here, it should take about 3.5 seconds for the buffer to empty.
	errs	+= service_ioctl_set_reg_list(fd, list_3);
	errs	+= service_ioctl_set_reg_list(fd, clock_disable);
	sent	= ao20_write(fd, buf, sizeof(buf));

	if (sent != sizeof(buf))
	{
		errs++;
		printf(	"ERROR: ao20_write(): send %ld, expected %ld)\n",
				(long) sent,
				(long) sizeof(buf));
	}

	errs	+= service_ioctl_set_reg_list(fd, clock_enable);
	os_sleep_ms(3000);
	ret		= ao20_ioctl(fd, AO20_IOCTL_BUFFER_STATUS,&_3_ne);
	errs	+= ret ? 1 : 0;
	os_sleep_ms(1000);
	ret		= ao20_ioctl(fd, AO20_IOCTL_BUFFER_STATUS,&_4_e);
	errs	+= ret ? 1 : 0;
	errs	+= service_ioctl_set_reg_list(fd, clock_disable);

	// Now verify all of the results.

	if (errs)
	{
	}
	else if (_1_ne == AO20_BUFFER_STATUS_EMPTY)
	{
		errs++;
		printf("FAIL <--- (Test 1 results are EMPTY.)\n");
	}
	else if (_2_e != AO20_BUFFER_STATUS_EMPTY)
	{
		errs++;
		printf("FAIL <--- (Test 2 results are not EMPTY.)\n");
	}
	else if (_3_ne == AO20_BUFFER_STATUS_EMPTY)
	{
		errs++;
		printf("FAIL <--- (Test 3 results are EMPTY.)\n");
	}
	else if (_4_e != AO20_BUFFER_STATUS_EMPTY)
	{
		errs++;
		printf("FAIL <--- (Test 4 results are not EMPTY.)\n");
	}

	errs	+= ao20_initialize(fd, -1, 0);
	return(errs);
}




/******************************************************************************
*
*	Function:	channel_sel_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AO20_IOCTL_CHANNEL_SEL.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int channel_sel_test(int fd)
{
	int	errs	= 0;

	gsc_label("AO20_IOCTL_CHANNEL_SEL");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


