// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO20/16AO20_Linux_2.x.x.x_GSC_DN/utils/util_query.c $
// $Rev: 52760 $
// $Date: 2023-04-05 15:06:22 -0500 (Wed, 05 Apr 2023) $

// 16AO20: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao20_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AO20_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	ao20_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AO20_QUERY_AUTO_CAL_MS:		ptr	= "Auto-Cal Duration";		break;
		case AO20_QUERY_CHANNEL_MASK:		ptr	= "Channel Mask";			break;
		case AO20_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";			break;
		case AO20_QUERY_CHANNEL_QTY:		ptr	= "Installed Channels";		break;
		case AO20_QUERY_COUNT:				ptr	= "Query Options";			break;
		case AO20_QUERY_DEVICE_TYPE:		ptr	= "Device Type";			break;
		case AO20_QUERY_EXT_CLK_IN_POL:		ptr	= "Ext Clk In Polarity";	break;
		case AO20_QUERY_EXT_TRIG_IN_POL:	ptr	= "Ext Trig In Polarity";	break;
		case AO20_QUERY_EXT_TRIG_OUT_POL:	ptr	= "Ext Trig Out Polarity";	break;
		case AO20_QUERY_FIFO_SIZE:			ptr	= "FIFO Size";				break;
		case AO20_QUERY_FILTER:				ptr	= "Filter";					break;
		case AO20_QUERY_FREF_DEFAULT:		ptr	= "Fref Default";			break;
		case AO20_QUERY_FSAMP_MAX:			ptr	= "Fsamp Max";				break;
		case AO20_QUERY_FSAMP_MIN:			ptr	= "Fsamp Min";				break;
		case AO20_QUERY_INIT_MS:			ptr	= "Init. Duration";			break;
		case AO20_QUERY_NCLK_MASK:			ptr	= "Nclk Mask";				break;
		case AO20_QUERY_NCLK_MAX:			ptr	= "Nclk Max";				break;
		case AO20_QUERY_NCLK_MIN:			ptr	= "Nclk Min";				break;
		case AO20_QUERY_NRATE_MASK:			ptr	= "Nrate Mask";				break;
		case AO20_QUERY_NRATE_MAX:			ptr	= "Nrate Max";				break;
		case AO20_QUERY_NRATE_MIN:			ptr	= "Nrate Min";				break;
		case AO20_QUERY_RANGE:				ptr	= "Voltage Range";			break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= ao20_ioctl(fd, AO20_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AO20_QUERY_CHANNEL_MAX:
		case AO20_QUERY_CHANNEL_QTY:
		case AO20_QUERY_COUNT:
		case AO20_QUERY_NCLK_MAX:
		case AO20_QUERY_NCLK_MIN:
		case AO20_QUERY_NRATE_MAX:
		case AO20_QUERY_NRATE_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case AO20_QUERY_CHANNEL_MASK:
		case AO20_QUERY_NCLK_MASK:
		case AO20_QUERY_NRATE_MASK:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case AO20_QUERY_FREF_DEFAULT:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case AO20_QUERY_AUTO_CAL_MS:
		case AO20_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case AO20_QUERY_FSAMP_MAX:
		case AO20_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case AO20_QUERY_EXT_CLK_IN_POL:
		case AO20_QUERY_EXT_TRIG_IN_POL:
		case AO20_QUERY_EXT_TRIG_OUT_POL:

			sprintf(buf, "%s", set ? "Supported" : "Not Supported");
			break;

		case AO20_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case AO20_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_16AO20)
			{
				strcpy(buf, "16AO20");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case AO20_QUERY_FILTER:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case AO20_FILTER_NONE:		strcpy(buf, "None");	break;
				case AO20_FILTER_10KHZ:		strcpy(buf, "10K Hz");	break;
				case AO20_FILTER_100KHZ:	strcpy(buf, "100K Hz");	break;
			}

			break;

		case AO20_QUERY_RANGE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case AO20_RANGE_2_5:	strcpy(buf, "+- 2.5 Volts");	break;
				case AO20_RANGE_5:		strcpy(buf, "+- 5 Volts");		break;
				case AO20_RANGE_10:		strcpy(buf, "+- 10 Volts");		break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


